package controller_test

import (
	"testing"

	e "galaxy/error"

	"github.com/google/uuid"

	"galaxy/game/internal/model/game"

	"github.com/stretchr/testify/assert"
)

func TestShipGroupSend(t *testing.T) {
	c, g := newCache()
	// group #1 - in_orbit, free to upgrade
	assert.NoError(t, c.CreateShips(Race_0_idx, ShipType_Cruiser, R0_Planet_0_num, 10))
	// group #2 - in_space
	assert.NoError(t, c.CreateShips(Race_0_idx, ShipType_Cruiser, R0_Planet_0_num, 1))
	c.ShipGroup(1).StateInSpace = &InSpace
	// group #3 - in_orbit, unmovable
	g.ShipClassCreate(Race_0.Name, "Fortress", 0, 50, 30, 100, 0)
	assert.NoError(t, c.CreateShips(Race_0_idx, "Fortress", R0_Planet_0_num, 1))

	shiplessRace := "Shipless"
	ri, _ := c.AddRace(shiplessRace)
	assert.NoError(t, c.ShipClassCreate(ri, "Drone", 1, 0, 0, 0, 0))
	sgi := c.CreateShipsUnsafe_T(ri, c.MustShipClass(ri, "Drone").ID, R0_Planet_0_num, 1)

	assert.ErrorContains(t,
		g.ShipGroupSend(UnknownRace, c.ShipGroup(0).ID, 2),
		e.GenericErrorText(e.ErrInputUnknownRace))
	assert.ErrorContains(t,
		g.ShipGroupSend(Race_Extinct.Name, c.ShipGroup(0).ID, 2),
		e.GenericErrorText(e.ErrRaceExinct))
	assert.ErrorContains(t,
		g.ShipGroupSend(Race_0.Name, uuid.New(), 2),
		e.GenericErrorText(e.ErrInputEntityNotExists))
	assert.ErrorContains(t,
		g.ShipGroupSend(Race_0.Name, c.ShipGroup(0).ID, 222),
		e.GenericErrorText(e.ErrInputEntityNotExists))
	assert.ErrorContains(t,
		g.ShipGroupSend(Race_0.Name, c.ShipGroup(1).ID, 1),
		e.GenericErrorText(e.ErrShipsBusy))
	assert.ErrorContains(t,
		g.ShipGroupSend(shiplessRace, c.ShipGroup(sgi).ID, 2),
		e.GenericErrorText(e.ErrSendShipOwnerHasNoPlanets))
	assert.ErrorContains(t,
		g.ShipGroupSend(Race_0.Name, c.ShipGroup(2).ID, 2),
		e.GenericErrorText(e.ErrSendShipHasNoDrives))
	assert.ErrorContains(t,
		g.ShipGroupSend(Race_0.Name, c.ShipGroup(0).ID, 3),
		e.GenericErrorText(e.ErrSendUnreachableDestination))

	assert.NoError(t, g.ShipGroupSend(Race_0.Name, c.ShipGroup(0).ID, R0_Planet_2_num)) // send #0
	assert.Equal(t, game.StateLaunched, c.ShipGroup(0).State())
	assert.NotNil(t, c.ShipGroup(0).StateInSpace)
	assert.Nil(t, c.ShipGroup(0).StateInSpace.X)
	assert.Nil(t, c.ShipGroup(0).StateInSpace.Y)

	assert.NoError(t, g.ShipGroupSend(Race_0.Name, c.ShipGroup(0).ID, R0_Planet_0_num)) // un-send #0
	assert.Equal(t, game.StateInOrbit, c.ShipGroup(0).State())
}
